@@ -41,6 +41,7 @@ from autobahn.wamp import exception
 from autobahn.wamp.exception import ApplicationError, ProtocolError, SessionNotReady, SerializationError
 from autobahn.wamp.interfaces import ISession, IPayloadCodec, IAuthenticator  # noqa
 from autobahn.wamp.types import SessionDetails, CloseDetails, EncodedPayload
+from autobahn.exception import PayloadExceededError
 from autobahn.wamp.request import \
     Publication, \
     Subscription, \
@@ -93,7 +94,8 @@ class BaseSession(ObservableMixin):
 
         # mapping of WAMP error URIs to exception classes
         self._uri_to_ecls = {
-            ApplicationError.INVALID_PAYLOAD: SerializationError
+            ApplicationError.INVALID_PAYLOAD: SerializationError,
+            ApplicationError.PAYLOAD_SIZE_EXCEEDED: PayloadExceededError,
         }
 
         # session authentication information
@@ -1066,6 +1068,12 @@ class ApplicationSession(BaseSession):
                                     reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.INVALID_PAYLOAD,
                                                           args=[u'success return value from invoked procedure "{0}" could not be serialized: {1}'.format(registration.procedure, e)])
                                     self._transport.send(reply)
+                                except PayloadExceededError as e:
+                                    # the application-level payload returned from the invoked procedure, when serialized and framed
+                                    # for the transport, exceeds the transport message/frame size limit
+                                    reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.PAYLOAD_SIZE_EXCEEDED,
+                                                          args=[u'success return value from invoked procedure "{0}" exceeds transport size limit: {1}'.format(registration.procedure, e)])
+                                    self._transport.send(reply)
 
                             def error(err):
                                 del self._invocations[msg.request]
@@ -1096,6 +1104,13 @@ class ApplicationSession(BaseSession):
                                     reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.INVALID_PAYLOAD,
                                                           args=[u'error return value from invoked procedure "{0}" could not be serialized: {1}'.format(registration.procedure, e)])
                                     self._transport.send(reply)
+                                except PayloadExceededError as e:
+                                    # the application-level payload returned from the invoked procedure, when serialized and framed
+                                    # for the transport, exceeds the transport message/frame size limit
+                                    reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.PAYLOAD_SIZE_EXCEEDED,
+                                                          args=[u'success return value from invoked procedure "{0}" exceeds transport size limit: {1}'.format(registration.procedure, e)])
+                                    self._transport.send(reply)
+
                                 # we have handled the error, so we eat it
                                 return None
 
@@ -1357,6 +1372,12 @@ class ApplicationSession(BaseSession):
         assert(args is None or type(args) in (list, tuple))
         assert(kwargs is None or type(kwargs) == dict)
 
+        message.check_or_raise_uri(topic,
+                                   message='{}.publish()'.format(self.__class__.__name__),
+                                   strict=False,
+                                   allow_empty_components=False,
+                                   allow_none=False)
+
         options = kwargs.pop('options', None)
         if options and not isinstance(options, types.PublishOptions):
             raise Exception("options must be of type a.w.t.PublishOptions")
@@ -1452,6 +1473,12 @@ class ApplicationSession(BaseSession):
             raise exception.TransportLost()
 
         def _subscribe(obj, fn, topic, options):
+            message.check_or_raise_uri(topic,
+                                       message='{}.subscribe()'.format(self.__class__.__name__),
+                                       strict=False,
+                                       allow_empty_components=True,
+                                       allow_none=False)
+
             request_id = self._request_id_gen.next()
             on_reply = txaio.create_future()
             handler_obj = Handler(fn, obj, options.details_arg if options else None)
@@ -1543,6 +1570,12 @@ class ApplicationSession(BaseSession):
         assert(args is None or type(args) in (list, tuple))
         assert(kwargs is None or type(kwargs) == dict)
 
+        message.check_or_raise_uri(procedure,
+                                   message='{}.call()'.format(self.__class__.__name__),
+                                   strict=False,
+                                   allow_empty_components=False,
+                                   allow_none=False)
+
         options = kwargs.pop('options', None)
         if options and not isinstance(options, types.CallOptions):
             raise Exception("options must be of type a.w.t.CallOptions")
@@ -1646,6 +1679,12 @@ class ApplicationSession(BaseSession):
             raise exception.TransportLost()
 
         def _register(obj, fn, procedure, options):
+            message.check_or_raise_uri(procedure,
+                                       message='{}.register()'.format(self.__class__.__name__),
+                                       strict=False,
+                                       allow_empty_components=True,
+                                       allow_none=False)
+
             request_id = self._request_id_gen.next()
             on_reply = txaio.create_future()
             endpoint_obj = Endpoint(fn, obj, options.details_arg if options else None)
@@ -1689,7 +1728,7 @@ class ApplicationSession(BaseSession):
                             regopts = pat.options or options
                             on_replies.append(_register(endpoint, proc, _uri, regopts))
 
-            # XXX neds coverage
+            # XXX needs coverage
             return txaio.gather(on_replies, consume_exceptions=True)
 
     def _unregister(self, registration):
